@@ -8,6 +8,11 @@ constants that are used in multiple other modules
 import enum
 import numpy as np
 
+from numbers import Real, Integral
+from pathlib import PosixPath, WindowsPath
+
+from typing import Dict
+
 
 class Options(str, enum.Enum):
     """
@@ -25,14 +30,12 @@ class Options(str, enum.Enum):
     STEPBACK_STRAT = 'stepback_strategy'  #: method to use for stepback
     THETA_MAX = 'theta_max'  #: maximal fraction of step that would hit bounds
     DELTA_INIT = 'delta_init'  #: initial trust region radius
-    MU = 'mu'  # acceptance threshold for trust region ratio
-    ETA = 'eta'  # trust region increase threshold for trust region ratio
-    GAMMA1 = 'gamma1'  # factor by which trust region radius will be decreased
-    GAMMA2 = 'gamma2'  # factor by which trust region radius will be increased
-    REFINE_STEPBACK = 'refine_stepback'  # whether stepbacks are refined via
-    # optimization
-    SCALED_GRADIENT = 'scaled_gradient'  # whether scaled gradient should be
-    # added to the set of possible stepback proposals
+    MU = 'mu'  #: acceptance threshold for trust region ratio
+    ETA = 'eta'  #: trust region increase threshold for trust region ratio
+    GAMMA1 = 'gamma1'  #: factor by which trust region radius will be decreased
+    GAMMA2 = 'gamma2'  #: factor by which trust region radius will be increased
+    HISTORY_FILE = 'history_file'  #: when set, statistics for each start will
+    # be saved to the specified file
 
 
 class SubSpaceDim(str, enum.Enum):
@@ -55,6 +58,7 @@ class StepBackStrategy(str, enum.Enum):
     TRUNCATE = 'truncate'  #: truncate step at boundary and re-solve
     # restricted subproblem
     MIXED = 'mixed'  #: mix reflections and truncations
+    REFINE = 'refine'  #: perform optimization to refine step
 
 
 DEFAULT_OPTIONS = {
@@ -73,8 +77,7 @@ DEFAULT_OPTIONS = {
     Options.ETA: 0.75,  # [NodedalWright2006]
     Options.GAMMA1: 1/4,  # [NodedalWright2006]
     Options.GAMMA2: 2,  # [NodedalWright2006]
-    Options.REFINE_STEPBACK: False,
-    Options.SCALED_GRADIENT: False,
+    Options.HISTORY_FILE: None,
 }
 
 
@@ -93,3 +96,44 @@ class ExitFlag(int, enum.Enum):
     FTOL = 1  #: Converged according to fval difference
     XTOL = 2  #: Converged according to x difference
     GTOL = 3  #: Converged according to gradient norm
+
+
+def validate_options(options: Dict):
+    """Check if the chosen options are valid"""
+    expected_types = {
+        Options.MAXITER: Integral,
+        Options.MAXTIME: Real,
+        Options.FATOL: Real,
+        Options.FRTOL: Real,
+        Options.XTOL: Real,
+        Options.GATOL: Real,
+        Options.GRTOL: Real,
+        Options.SUBSPACE_DIM: (SubSpaceDim, str),
+        Options.STEPBACK_STRAT: (StepBackStrategy, str),
+        Options.THETA_MAX: Real,
+        Options.DELTA_INIT: Real,
+        Options.MU: Real,
+        Options.ETA: Real,
+        Options.GAMMA1: Real,
+        Options.GAMMA2: Real,
+        Options.HISTORY_FILE: (str, PosixPath, WindowsPath),
+    }
+    for option_key, option_value in options.items():
+        try:
+            option = Options(option_key)
+        except ValueError:
+            raise ValueError(f'{option_key} is not a valid options field.')
+
+        if option_key is Options.SUBSPACE_DIM:
+            option_value = SubSpaceDim(option_value)
+
+        if option_key is Options.STEPBACK_STRAT:
+            option_value = StepBackStrategy(option_value)
+
+        expected_type = expected_types[option]
+        if not isinstance(option_value, expected_type):
+            if expected_type == Integral and int(option_value) == option_value:
+                continue
+            raise TypeError(f'Type mismatch for option {option_key}. '
+                            f'Expected {expected_type} but got '
+                            f'{type(option_value)}')
