@@ -26,9 +26,10 @@ class HessianApproximation:
             according to the user-provided objective function
         """
         self._hess: np.ndarray = np.empty(0)
+        self._diff: np.ndarray = np.empty(0)
         self.init_with_hess = init_with_hess
 
-    def init_mat(self, dim: int, hess: Optional[np.ndarray] = None):
+    def init_mat(self, dim: int, hess: Optional[np.ndarray] = None) -> None:
         """
         Initializes this approximation instance and checks the dimensionality
 
@@ -46,24 +47,38 @@ class HessianApproximation:
                                  f'dimension, was {hess.shape[0]}, '
                                  f'but should be {dim}.')
             self._hess = hess.copy()
+        self._diff = np.zeros_like(self._hess)
 
     def get_mat(self) -> np.ndarray:
         """
         Getter for the Hessian approximation
-        :return:
-        """
-        return self._hess
 
-    def set_mat(self, mat: np.ndarray):
-        """
-        Getter for the Hessian approximation
         :return:
+            Hessian approximation
+        """
+        return self._hess.copy()
+
+    def get_diff(self) -> np.ndarray:
+        """
+        Getter for the Hessian approximation update
+
+        :return:
+            Hessian approximation update
+        """
+        return self._diff.copy()
+
+    def set_mat(self, mat: np.ndarray) -> None:
+        """
+        Setter for the Hessian approximation
+
+        :param mat:
+            Hessian approximation
         """
         if mat.shape != self._hess.shape:
             raise ValueError('Passed matrix had inconsistent '
                              f'shape, was {mat.shape}, '
                              f'but should be {self._hess.shape}.')
-        self._hess = mat
+        self._hess = mat.copy()
 
     @property
     def requires_resfun(self):
@@ -73,14 +88,34 @@ class HessianApproximation:
     def requires_hess(self):
         return False  # pragma: no cover
 
+    def _update_hess_and_store_diff(self, hess):
+        self._diff = hess - self._hess
+        self._hess = hess.copy()
+
 
 class IterativeHessianApproximation(HessianApproximation):
     """
     Iterative update schemes that only use s and y values for update.
     """
-    def update(self, s, y):
+    def update(self, s: np.ndarray, y: np.ndarray) -> None:
+        """
+        Update the Hessian approximation
+
+        :param s:
+            step in optimization variables
+
+        :param y:
+            step in gradient
+        """
+        self._compute_update(s, y)
+        self._apply_update()
+
+    def _compute_update(self, s: np.ndarray, y: np.ndarray) -> None:
         raise NotImplementedError()  # pragma: no cover
 
+    def _apply_update(self) -> None:
+        self._hess += self._diff
+
 
 class Broyden(IterativeHessianApproximation):
     """
@@ -96,19 +131,25 @@ class Broyden(IterativeHessianApproximation):
     :parameter phi:
         convex combination parameter interpolating between BFGS (phi==0) and
         DFP (phi==1).
+    :parameter enforce_curv_cond:
+        boolean that controls whether the employed broyden class update
+        should attempt to preserve positive definiteness. If set to True,
+        updates from steps that violate the curvature condition will be
+        discarded.
     """
-    def __init__(self, phi: float, init_with_hess: Optional[bool] = False):
+    def __init__(self, phi: float, init_with_hess: Optional[bool] = False,
+                 enforce_curv_cond: Optional[bool] = True):
         self.phi = phi
+        self.enforce_curv_cond = enforce_curv_cond
         if phi < 0 or phi > 1:
             warnings.warn('Setting phi to values outside the interval [0, 1]'
                           'will not guarantee that positive definiteness is '
                           'preserved during updating.')
         super(Broyden, self).__init__(init_with_hess)
 
-    def update(self, s, y):
-        if y.T.dot(s) <= 0:
-            return
-        self._hess += broyden_class_update(y, s, self._hess, self.phi)
+    def _compute_update(self, s: np.ndarray, y: np.ndarray):
+        self._diff = broyden_class_update(y, s, self._hess, self.phi,
+                                          self.enforce_curv_cond)
 
 
 class BFGS(Broyden):
@@ -118,8 +159,10 @@ class BFGS(Broyden):
 
     This scheme only works with a function that returns (fval, grad)
     """
-    def __init__(self, init_with_hess: Optional[bool] = False):
-        super(BFGS, self).__init__(phi=0.0, init_with_hess=init_with_hess)
+    def __init__(self, init_with_hess: Optional[bool] = False,
+                 enforce_curv_cond: Optional[bool] = True):
+        super(BFGS, self).__init__(phi=0.0, init_with_hess=init_with_hess,
+                                   enforce_curv_cond=enforce_curv_cond)
 
 
 class DFP(Broyden):
@@ -129,8 +172,10 @@ class DFP(Broyden):
 
     This scheme only works with a function that returns (fval, grad)
     """
-    def __init__(self, init_with_hess: Optional[bool] = False):
-        super(DFP, self).__init__(phi=1.0, init_with_hess=init_with_hess)
+    def __init__(self, init_with_hess: Optional[bool] = False,
+                 enforce_curv_cond: Optional[bool] = True):
+        super(DFP, self).__init__(phi=1.0, init_with_hess=init_with_hess,
+                                  enforce_curv_cond=enforce_curv_cond)
 
 
 class SR1(IterativeHessianApproximation):
@@ -142,13 +187,15 @@ class SR1(IterativeHessianApproximation):
 
     This scheme only works with a function that returns (fval, grad)
     """
-    def update(self, s, y):
+    def _compute_update(self, s: np.ndarray, y: np.ndarray):
         z = y - self._hess.dot(s)
         d = z.T.dot(s)
 
         # [NocedalWright2006] (6.26) reject if update degenerate
-        if np.abs(d) >= 1e-8 * norm(s) * norm(z):
-            self._hess += np.outer(z, z.T) / d
+        if np.abs(d) >= np.sqrt(np.spacing(1)) * norm(s) * norm(z):
+            self._diff = np.outer(z, z.T) / d
+        else:
+            self._diff = np.zeros_like(self._hess)
 
 
 class BG(IterativeHessianApproximation):
@@ -160,8 +207,9 @@ class BG(IterativeHessianApproximation):
 
     This scheme only works with a function that returns (fval, grad)
     """
-    def update(self, s, y):
-        self._hess += np.outer(y - self._hess.dot(s), s.T) / s.T.dot(s)
+    def _compute_update(self, s: np.ndarray, y: np.ndarray):
+        z = y - self._hess.dot(s)
+        self._diff = np.outer(z, s.T) / s.T.dot(s)
 
 
 class BB(IterativeHessianApproximation):
@@ -173,14 +221,16 @@ class BB(IterativeHessianApproximation):
 
     This scheme only works with a function that returns (fval, grad)
     """
-    def update(self, s, y):
+    def update(self, s: np.ndarray, y: np.ndarray) -> None:
         b = y.T.dot(s)
+        z = y - self._hess.dot(s)
         if b <= 0:
-            return
-        self._hess += np.outer(y - self._hess.dot(s), s.T) / b
+            self._diff = np.zeros_like(self._hess)
+        else:
+            self._diff = np.outer(z, s.T) / b
 
 
-class HybridSwitchApproximation(HessianApproximation):
+class HybridApproximation(HessianApproximation):
     def __init__(self, happ: IterativeHessianApproximation = BFGS()):
         """
         Create a Hybrid Hessian update strategy that switches between an
@@ -190,53 +240,87 @@ class HybridSwitchApproximation(HessianApproximation):
             Iterative Hessian Approximation
         """
         self.hessian_update = happ
-        super(HybridSwitchApproximation, self).__init__()
+        super(HybridApproximation, self).__init__()
 
     def init_mat(self, dim: int, hess: Optional[np.ndarray] = None):
         self.hessian_update.init_mat(dim, hess)
-        super(HybridSwitchApproximation, self).init_mat(dim, hess)
-
-    def get_mat(self) -> np.ndarray:
-        return self.hessian_update.get_mat()
+        super(HybridApproximation, self).init_mat(dim, hess)
 
     def requires_hess(self):
         return True  # pragma: no cover
 
 
+class HybridSwitchApproximation(HybridApproximation):
+    def _switched_update(self, s: np.ndarray, y: np.ndarray,
+                         hess: np.ndarray):
+        self.hessian_update.update(s, y)
+        if self._switched:
+            new_hess = self.hessian_update.get_mat()
+        else:
+            new_hess = hess
+        self._update_hess_and_store_diff(new_hess)
+
+
 class HybridFixed(HybridSwitchApproximation):
     def __init__(self,
                  happ: IterativeHessianApproximation = BFGS(),
                  switch_iteration: Optional[int] = 20):
         """
-        Switch from a dynamic approximation that to the user provided
-        iterative scheme after a fixed number of iterations. The iterative
-        scheme is initialized and updated from the beginning, but only
+        Switch from a dynamic approximation to the user provided iterative
+        scheme after a fixed number of successive iterations without
+        trust-region update. The switching is non-reversible. The iterative
+        scheme is initialized and updated rom the beginning, but only
         employed after the specified number of iterations.
 
         This scheme only works with a function that returns (fval, grad, hess)
 
         :param switch_iteration:
-            Iteration after which this approximation is used
+            Number of iterations without trust region update after which
+            switch occurs.
         """
         self.switch_iteration: int = switch_iteration
         super(HybridFixed, self).__init__(happ)
         self._switched = False
 
-    def update(self, s, y, hess, iter_since_tr_update):
-        self.hessian_update.update(s, y)
-        self._hess = hess
-        if self._switched:
-            return
-        self._switched = iter_since_tr_update >= self.switch_iteration
-
-    def get_mat(self) -> np.ndarray:
-        if self._switched:
-            return self.hessian_update.get_mat()
-        else:
-            return self._hess
+    def update(self, s: np.ndarray, y: np.ndarray, hess: np.ndarray,
+               iter_since_tr_update: int):
+        self._switched_update(s, y, hess)
+        if not self._switched:
+            self._switched = iter_since_tr_update >= self.switch_iteration
 
 
-class FX(HybridSwitchApproximation):
+class HybridFraction(HybridSwitchApproximation):
+    def __init__(self,
+                 happ: IterativeHessianApproximation = BFGS(),
+                 switch_threshold: Optional[float] = 0.8):
+        """
+        Switch from a dynamic approximation to the user provided iterative
+        scheme as soon as the fraction of iterations where the step is
+        accepted but the trust region is not update exceeds the user provided
+        threshold.Threshold check is only performed after 25 iterations.
+        The switching is  non-reversible. The iterative scheme is
+        initialized and updated rom the beginning, but only employed after
+        the switching.
+
+        This scheme only works with a function that returns (fval, grad, hess)
+
+        :param switch_threshold:
+            Threshold for fraction of iterations where step is accepted but
+            trust region is not updated, which when exceeded triggers switch
+            of approximation.
+        """
+        self.switch_threshold: float = switch_threshold
+        super(HybridFraction, self).__init__(happ)
+        self._switched = False
+
+    def update(self, s: np.ndarray, y: np.ndarray, hess: np.ndarray,
+               tr_nonupdates: int, iterations: int):
+        self._switched_update(s, y, hess)
+        if not self._switched and iterations > 25:
+            self._switched = tr_nonupdates/iterations > self.switch_threshold
+
+
+class FX(HybridApproximation):
     def __init__(self,
                  happ: IterativeHessianApproximation = BFGS(),
                  hybrid_tol: Optional[float] = 0.2):
@@ -256,20 +340,46 @@ class FX(HybridSwitchApproximation):
         self.hybrid_tol = hybrid_tol
         super(FX, self).__init__(happ)
 
-    def update(self, delta, gamma, r, rprev, hess):
+    def update(self, delta: np.ndarray, gamma: np.ndarray,
+               r: np.ndarray, rprev: np.ndarray, hess: np.ndarray) -> None:
+        """
+        Update the Hessian approximation
+
+        :param delta:
+            step in optimization variables
+
+        :param gamma:
+            step in gradient
+
+        :param r:
+            residuals after current step
+
+        :param rprev:
+            residuals befor current step
+
+        :param hess:
+            user-provided (Gauss-Newton) Hessian approximation
+        """
         # Equation (3.5)
         ratio = (rprev.dot(rprev) - r.dot(r))/rprev.dot(rprev)
         if ratio >= self.hybrid_tol:
+            self._diff = hess - self.hessian_update.get_mat()
             self.hessian_update.set_mat(hess)
         else:
             self.hessian_update.update(delta, gamma)
+            self._diff = self.hessian_update.get_diff()
 
-    def requires_resfun(self):
+    def get_mat(self) -> np.ndarray:
+        return self.hessian_update.get_mat()
+
+    @property
+    def requires_resfun(self) -> bool:
         return True  # pragma: no cover
 
 
 class StructuredApproximation(HessianApproximation):
-    def __init__(self, phi: Optional[float] = 0.0):
+    def __init__(self, phi: Optional[float] = 0.0,
+                 enforce_curv_cond: Optional[bool] = True):
         """
         This is the base class for structured secant methods (SSM). SSMs
         approximate the hessian by combining the Gauss-Newton component C(x)
@@ -279,9 +389,17 @@ class StructuredApproximation(HessianApproximation):
         :parameter phi:
             convex combination parameter interpolating between BFGS (phi==0)
             and DFP (phi==1) update schemes.
+
+        :parameter enforce_curv_cond:
+            boolean that controls whether the employed broyden class update
+            should attempt to preserve positive definiteness. If set to True,
+            updates from steps that violate the curvature condition will be
+            discarded.
         """
         self.A: np.ndarray = np.empty(0)
         self.phi = phi
+        self.enforce_curv_cond = enforce_curv_cond
+        self._structured_diff = np.empty(0)
         if phi < 0 or phi > 1:
             warnings.warn('Setting phi to values outside the interval [0, 1]'
                           'will not guarantee that positive definiteness is '
@@ -290,18 +408,38 @@ class StructuredApproximation(HessianApproximation):
 
     def init_mat(self, dim: int, hess: Optional[np.ndarray] = None):
         self.A = np.eye(dim) * np.spacing(1)
+        self._structured_diff = np.zeros_like(self.A)
         super(StructuredApproximation, self).init_mat(dim, hess)
 
     def update(self, s: np.ndarray, y: np.ndarray, r: np.ndarray,
-               hess: np.ndarray, yb: np.ndarray):
+               hess: np.ndarray, yb: np.ndarray) -> None:
+        """
+        Update the structured approximation
+
+        :param s:
+            step in optimization parameters
+        :param y:
+            step in gradient parameters
+        :param r:
+            residual vector
+        :param hess:
+            user-provided (Gauss-Newton) Hessian approximation
+        :param yb:
+            approximation to A*s, where A is structured approximation matrix
+        """
         raise NotImplementedError()  # pragma: no cover
 
+    @property
     def requires_resfun(self):
         return True  # pragma: no cover
 
+    @property
     def requires_hess(self):
         return True  # pragma: no cover
 
+    def get_structured_diff(self) -> np.ndarray:
+        return self._structured_diff
+
 
 class SSM(StructuredApproximation):
     """
@@ -313,15 +451,18 @@ class SSM(StructuredApproximation):
     """
 
     def update(self, s: np.ndarray, y: np.ndarray, r: np.ndarray,
-               hess: np.ndarray, yb: np.ndarray):
+               hess: np.ndarray, yb: np.ndarray) -> None:
         # B^S = A + C(x_+)
         Bs = hess + self.A
         # y^S = y^# + C(x_+)*s
         ys = yb + hess.dot(s)
         # Equation (13)
-        self.A += broyden_class_update(ys, s, Bs, phi=self.phi)
+        self._structured_diff = broyden_class_update(
+            ys, s, Bs, phi=self.phi, enforce_curv_cond=self.enforce_curv_cond
+        )
+        self.A += self._structured_diff
         # B_+ = C(x_+) + A + BFGS update A (=A_+)
-        self._hess = hess + self.A
+        self._update_hess_and_store_diff(hess + self.A)
 
 
 class TSSM(StructuredApproximation):
@@ -334,19 +475,23 @@ class TSSM(StructuredApproximation):
     """
 
     def update(self, s: np.ndarray, y: np.ndarray, r: np.ndarray,
-               hess: np.ndarray, yb: np.ndarray):
+               hess: np.ndarray, yb: np.ndarray) -> None:
         # Equation (2.7)
         Bs = hess + norm(r) * self.A
         # Equation (2.6)
         ys = hess.dot(s) + yb
         # Equation (2.10)
-        self.A += broyden_class_update(ys, s, Bs, phi=self.phi)/norm(r)
+        self._structured_diff = broyden_class_update(
+            ys, s, Bs, phi=self.phi, enforce_curv_cond=self.enforce_curv_cond
+        )/norm(r)
+        self.A += self._structured_diff
         # Equation (2.9)
-        self._hess = hess + norm(r) * self.A
+        self._update_hess_and_store_diff(hess + norm(r) * self.A)
 
 
 class GNSBFGS(StructuredApproximation):
-    def __init__(self, hybrid_tol: float = 1e-6):
+    def __init__(self, hybrid_tol: float = 1e-6,
+                 enforce_curv_cond: bool = True):
         """
         Hybrid Gauss-Newton Structured BFGS method as introduced by
         [Zhou & Chen 2010](https://doi.org/10.1137/090748470),
@@ -359,23 +504,33 @@ class GNSBFGS(StructuredApproximation):
             switching tolerance that controls switching between update methods
         """
         self.hybrid_tol: float = hybrid_tol
-        super(GNSBFGS, self).__init__(phi=0.0)
+        super(GNSBFGS, self).__init__(phi=0.0,
+                                      enforce_curv_cond=enforce_curv_cond)
 
     def update(self, s: np.ndarray, y: np.ndarray, r: np.ndarray,
-               hess: np.ndarray, yb: np.ndarray):
+               hess: np.ndarray, yb: np.ndarray) -> None:
         # Equation (2.1)
         ratio = yb.T.dot(s)/s.dot(s)
         if ratio > self.hybrid_tol:
             # Equation (2.3)
-            self.A += broyden_class_update(yb, s, self.A, phi=self.phi)
+            self._structured_diff = broyden_class_update(
+                yb, s, self.A, phi=self.phi,
+                enforce_curv_cond=self.enforce_curv_cond
+            )
+            self.A += self._structured_diff
             # Equation (2.2)
-            self._hess = hess + self.A
+            self._update_hess_and_store_diff(hess + self.A)
         else:
             # Equation (2.2)
-            self._hess = hess + norm(r) * np.eye(len(y))
+            self._structured_diff = np.zeros_like(self.A)
+            self._update_hess_and_store_diff(hess + norm(r) * np.eye(len(y)))
 
 
-def broyden_class_update(y, s, mat, phi=0.0):
+def broyden_class_update(y: np.ndarray,
+                         s: np.ndarray,
+                         mat: np.ndarray,
+                         phi: float = 0.0,
+                         enforce_curv_cond: bool = True) -> np.ndarray:
     """
     Scale free implementation of the broyden class update scheme.
 
@@ -388,13 +543,18 @@ def broyden_class_update(y, s, mat, phi=0.0):
     :param phi:
         convex combination parameter, interpolates between BFGS (phi=0) and
         DFP (phi=1).
+    :parameter enforce_curv_cond:
+        boolean that controls whether the employed broyden class update
+        should attempt to preserve positive definiteness. If set to True,
+        updates from steps that violate the curvature condition will be
+        discarded.
     """
     u = mat.dot(s)
     c = u.T.dot(s)
     b = y.T.dot(s)
 
-    if b <= 0:
-        return np.zeros(mat.shape)
+    if b <= 0 and enforce_curv_cond:
+        return np.zeros_like(mat)
 
     update = np.outer(y, y.T) / b - np.outer(u, u.T) / c
 
